@@ -93,6 +93,7 @@ class Device(object):
         super().__init__()
 
         self.name = name
+        self.connected = False
 
     def getNumber(self, propertyName):
         """
@@ -115,7 +116,7 @@ class Device(object):
         retDict = {}
         for prop in elementList:
             retDict[prop] = elementList[prop]['value']
-        self.log.info(f'Get number [{self.name}]: {retDict}')
+        # self.log.info(f'Get number [{self.name}]: {retDict}')
         return retDict
 
     def getText(self, propertyName):
@@ -139,7 +140,7 @@ class Device(object):
         retDict = {}
         for prop in elementList:
             retDict[prop] = elementList[prop]['value']
-        self.log.info(f'Get text   [{self.name}]: {retDict}')
+        # self.log.info(f'Get text   [{self.name}]: {retDict}')
         return retDict
 
     def getSwitch(self, propertyName):
@@ -163,7 +164,7 @@ class Device(object):
         retDict = {}
         for prop in elementList:
             retDict[prop] = elementList[prop]['value']
-        self.log.info(f'Get switch [{self.name}]: {retDict}')
+        # self.log.info(f'Get switch [{self.name}]: {retDict}')
         return retDict
 
     def getLight(self, propertyName):
@@ -187,7 +188,7 @@ class Device(object):
         retDict = {}
         for prop in elementList:
             retDict[prop] = elementList[prop]['value']
-        self.log.info(f'Get light  [{self.name}]: {retDict}')
+        # self.log.info(f'Get light  [{self.name}]: {retDict}')
         return retDict
 
     def getBlob(self, propertyName):
@@ -208,7 +209,7 @@ class Device(object):
             self.log.error('Property: {0} is not Blob'.format(iProperty['propertyType']))
             return
         elementList = iProperty['elementList']
-        self.log.info(f'Get blob   [{self.name}]')
+        # self.log.info(f'Get blob   [{self.name}]')
         return elementList[propertyName]
 
 
@@ -402,7 +403,7 @@ class Client(PyQt5.QtCore.QObject):
                 continue
             self.signals.removeDevice.emit(device)
             # self.signals.deviceDisconnected.emit(device)
-            self.log.warning('Remove device {0}'.format(device))
+            self.log.warning(f'Remove device [{device}]')
         self.devices = {}
         return True
 
@@ -416,10 +417,10 @@ class Client(PyQt5.QtCore.QObject):
         """
 
         self.connected = False
-        self.signals.serverDisconnected.emit(self.devices)
         self.clearParser()
+        self.signals.serverDisconnected.emit(self.devices)
         self.clearDevices(deviceName)
-        self.socket.close()
+        self.socket.abort()
 
         return True
 
@@ -450,14 +451,28 @@ class Client(PyQt5.QtCore.QObject):
         :param deviceName: name string of INDI device
         :return: success
         """
+        
+        # todo: do connected state for each device
 
         if not self.connected:
             return False
         if not deviceName:
             return False
+        if deviceName not in self.devices:
+            return False
+
+        con = self.devices[deviceName].getSwitch('CONNECTION')
+        if con['CONNECT'] == 'On':
+            self.log.warning(f'Device [{deviceName}] was connected at startup')
+            return False
+        else:
+            self.log.warning(f'Device [{deviceName}] unconnected - connect it now')
+
         suc = self.sendNewSwitch(deviceName=deviceName,
                                  propertyName='CONNECTION',
-                                 elements='CONNECT',
+                                 elements={'CONNECT': 'On',
+                                           'DISCONNECT': 'Off'
+                                           },
                                  )
         return suc
 
@@ -469,15 +484,25 @@ class Client(PyQt5.QtCore.QObject):
         :return: success
         """
 
+        # todo: do connected state for each device
+
         if not self.connected:
             return False
         if not deviceName:
             return False
         if deviceName not in self.devices:
             return False
+
+        con = self.devices[deviceName].getSwitch('CONNECTION')
+        if con['DISCONNECT'] == 'On':
+            self.log.warning(f'{deviceName} already disconnected')
+            return False
+
         suc = self.sendNewSwitch(deviceName=deviceName,
                                  propertyName='CONNECTION',
-                                 elements='DISCONNECT',
+                                 elements={'CONNECT': 'Off',
+                                           'DISCONNECT': 'On'
+                                           },
                                  )
         return suc
 
@@ -617,7 +642,7 @@ class Client(PyQt5.QtCore.QObject):
             elements = {elements: number}
         elementList = []
         for element in elements:
-            number = elements[element]
+            number = float(elements[element])
             elementList.append(
                 indiXML.oneNumber(number,
                                   indi_attr={'name': element}
@@ -722,7 +747,9 @@ class Client(PyQt5.QtCore.QObject):
         """
 
         if self.connected:
-            number = self.socket.write(indiCommand.toXML() + b'\n')
+            cmd = indiCommand.toXML()
+            self.log.debug(f"SendCmd: [{cmd.decode().lstrip('<').rstrip('/>')}]")
+            number = self.socket.write(cmd + b'\n')
             self.socket.flush()
             if number > 0:
                 return True
@@ -772,35 +799,19 @@ class Client(PyQt5.QtCore.QObject):
 
             # as a new blob vector does not  contain an initial value, we have to separate this
             if not isinstance(elt, indiXML.DefBLOB):
-
-                if elt.etype in ['defNumber',
-                                 'setNumber',
-                                 'oneNumber']:
-                    elementList[name]['value'] = float(elt.getValue())
-
-                elif elt.etype in ['defSwitch',
-                                   'oneSwitch',
-                                   'setSwitch']:
-                    elementList[name]['value'] = (elt.getValue() == 'On')
-
-                else:
-                    elementList[name]['value'] = elt.getValue()
+                elementList[name]['value'] = elt.getValue()
 
             # now all other attributes of element are stored
             for attr in elt.attr:
                 elementList[name][attr] = elt.attr[attr]
 
-            # if we don't set values, no connection signals
-            if defVector:
-                continue
-
             # send connected signals
-            if name == 'CONNECT' and elt.getValue() == 'On':
+            if name == 'CONNECT' and elt.getValue() == 'On' and chunk.attr['state'] == 'Ok':
                 self.signals.deviceConnected.emit(deviceName)
-                self.log.warning('Device {0} connected'.format(deviceName))
+                self.log.warning(f'Device [{deviceName}] connected')
             if name == 'DISCONNECT' and elt.getValue() == 'On':
                 self.signals.deviceDisconnected.emit(deviceName)
-                self.log.warning('Device {0} disconnected'.format(deviceName))
+                self.log.warning(f'Device [{deviceName}] disconnected')
 
         return True
 
@@ -844,7 +855,7 @@ class Client(PyQt5.QtCore.QObject):
         if deviceName not in self.devices:
             self.devices[deviceName] = Device(deviceName)
             self.signals.newDevice.emit(deviceName)
-            self.log.warning('New device {0}'.format(deviceName))
+            self.log.warning(f'New device [{deviceName}]')
 
         device = self.devices[deviceName]
         return device, deviceName
@@ -867,7 +878,7 @@ class Client(PyQt5.QtCore.QObject):
         if hasattr(device, iProperty):
             delattr(device, iProperty)
             self.signals.removeProperty.emit(deviceName, iProperty)
-            self.log.warning('New device [{0}] property {1}'.format(deviceName, iProperty))
+            self.log.warning(f'Device [{deviceName}] del property [{iProperty}]')
         return True
 
     def _setProperty(self, chunk=None, device=None, deviceName=None):
@@ -965,8 +976,7 @@ class Client(PyQt5.QtCore.QObject):
         :param chunk: raw indi XML element
         :return: success if it could be parsed
         """
-
-        self.log.debug('INDI XML chunk: {0}'.format(chunk))
+        self.log.debug(f'RecvCmd: [{chunk}]')
         if not self.connected:
             return False
 
